@@ -1,6 +1,7 @@
     package Math::MPC;
     use strict;
     use warnings;
+    use Math::MPFR;
 
     use constant  MPC_RNDNN => 0;
     use constant  MPC_RNDZN => 1;
@@ -77,6 +78,7 @@ Rmpc_set_default_prec2 Rmpc_get_default_prec2
 Rmpc_set_re_prec Rmpc_set_im_prec
 Rmpc_get_prec Rmpc_get_prec2 Rmpc_get_re_prec Rmpc_get_im_prec
 Rmpc_get_dc Rmpc_get_ldc
+Rmpc_get_DC Rmpc_get_LDC Rmpc_get_F128C
 RMPC_RE RMPC_IM RMPC_INEX_RE RMPC_INEX_IM
 Rmpc_clear Rmpc_clear_ptr Rmpc_clear_mpc
 Rmpc_deref4 Rmpc_get_str
@@ -103,7 +105,8 @@ Rmpc_set_f_ld Rmpc_set_q_ld Rmpc_set_z_ld Rmpc_set_ld_f Rmpc_set_ld_q Rmpc_set_l
 Rmpc_set_f_q Rmpc_set_q_f Rmpc_set_f_z Rmpc_set_z_f Rmpc_set_z_q Rmpc_set_q_z
 Rmpc_set_f_fr Rmpc_set_q_fr Rmpc_set_z_fr Rmpc_set_fr_f Rmpc_set_fr_q Rmpc_set_fr_z
 
-Rmpc_set_dc Rmpc_set_ldc
+Rmpc_set_dc Rmpc_set_ldc Rmpc_set_NV Rmpc_set_NV_NV
+Rmpc_set_DC Rmpc_set_LDC Rmpc_set_F128C
 
 Rmpc_add Rmpc_add_ui Rmpc_add_fr
 Rmpc_sub Rmpc_sub_ui Rmpc_ui_sub Rmpc_ui_ui_sub
@@ -123,7 +126,7 @@ Rmpc_set_nan Rmpc_swap
 Rmpc_mul_sj Rmpc_mul_ld Rmpc_mul_d Rmpc_div_sj Rmpc_sj_div Rmpc_div_ld Rmpc_ld_div Rmpc_div_d Rmpc_d_div
 );
 
-    our $VERSION = '1.04';
+    our $VERSION = '1.06';
     #$VERSION = eval $VERSION;
 
     DynaLoader::bootstrap Math::MPC $VERSION;
@@ -139,6 +142,7 @@ Rmpc_set_default_prec2 Rmpc_get_default_prec2
 Rmpc_set_re_prec Rmpc_set_im_prec
 Rmpc_get_prec Rmpc_get_prec2 Rmpc_get_re_prec Rmpc_get_im_prec
 Rmpc_get_dc Rmpc_get_ldc
+Rmpc_get_DC Rmpc_get_LDC Rmpc_get_F128C
 RMPC_RE RMPC_IM RMPC_INEX_RE RMPC_INEX_IM
 Rmpc_clear Rmpc_clear_ptr Rmpc_clear_mpc
 Rmpc_deref4 Rmpc_get_str
@@ -165,7 +169,8 @@ Rmpc_set_f_ld Rmpc_set_q_ld Rmpc_set_z_ld Rmpc_set_ld_f Rmpc_set_ld_q Rmpc_set_l
 Rmpc_set_f_q Rmpc_set_q_f Rmpc_set_f_z Rmpc_set_z_f Rmpc_set_z_q Rmpc_set_q_z
 Rmpc_set_f_fr Rmpc_set_q_fr Rmpc_set_z_fr Rmpc_set_fr_f Rmpc_set_fr_q Rmpc_set_fr_z
 
-Rmpc_set_dc Rmpc_set_ldc
+Rmpc_set_dc Rmpc_set_ldc Rmpc_set_NV Rmpc_set_NV_NV
+Rmpc_set_DC Rmpc_set_LDC Rmpc_set_F128C
 
 Rmpc_add Rmpc_add_ui Rmpc_add_fr
 Rmpc_sub Rmpc_sub_ui Rmpc_ui_sub Rmpc_ui_ui_sub
@@ -185,6 +190,14 @@ Rmpc_set_nan Rmpc_swap
 Rmpc_mul_sj Rmpc_mul_ld Rmpc_mul_d Rmpc_div_sj Rmpc_sj_div Rmpc_div_ld Rmpc_ld_div Rmpc_div_d Rmpc_d_div
 )]);
 
+$Math::MPC::NOK_POK = 0; # Set to 1 to allow warnings in new() and overloaded operations when
+                          # a scalar that has set both NOK (NV) and POK (PV) flags is encountered
+
+eval {require Math::Complex_C::Q;};
+
+if($@) {$Math::MPC::no_complex_c_q = $@}
+else   {$Math::MPC::no_complex_c_q = 0 }
+
 *TRmpc_out_str = \&Rmpc_out_str;
 *TRmpc_inp_str = \&Rmpc_inp_str;
 
@@ -203,16 +216,6 @@ Rmpc_mul_sj Rmpc_mul_ld Rmpc_mul_d Rmpc_div_sj Rmpc_sj_div Rmpc_div_ld Rmpc_ld_d
 *Rmpc_set_d_ld  = \&Rmpc_set_ld_ld;
 *Rmpc_set_ld_d  = \&Rmpc_set_ld_ld;
 
-*Rmpc_mul_sj  = \&Math::MPC::_mpc_mul_sj;
-*Rmpc_mul_ld  = \&Math::MPC::_mpc_mul_ld;
-*Rmpc_mul_d   = \&Math::MPC::_mpc_mul_d;
-*Rmpc_div_sj  = \&Math::MPC::_mpc_div_sj;
-*Rmpc_sj_div  = \&Math::MPC::_mpc_sj_div;
-*Rmpc_div_ld  = \&Math::MPC::_mpc_div_ld;
-*Rmpc_ld_div  = \&Math::MPC::_mpc_ld_div;
-*Rmpc_div_d   = \&Math::MPC::_mpc_div_d;
-*Rmpc_d_div   = \&Math::MPC::_mpc_d_div;
-
 # Beginning with mpc-1.0, mpc_mul_2exp and mpc_div_2exp
 # were renamed to mpc_mul_2ui and mpc_div_2ui.
 *Rmpc_mul_2exp = \&Rmpc_mul_2ui;
@@ -220,6 +223,11 @@ Rmpc_mul_sj Rmpc_mul_ld Rmpc_mul_d Rmpc_div_sj Rmpc_sj_div Rmpc_div_ld Rmpc_ld_d
 
 sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking
 
+sub overload_not_equiv {
+    return 0 if overload_equiv($_[0], $_[1], $_[2]);
+    return 1;
+}
+
 sub overload_string {
      return "(" . _get_str($_[0], 10, 0, Rmpc_get_default_rounding_mode()) . ")";
 
@@ -303,6 +311,24 @@ sub Rmpc_deref4 {
     return ($r_m, $r_e, $i_m, $i_e);
 }
 
+sub Rmpc_set_F128C {
+  if($Math::MPC::no_complex_c_q) {die "In Rmpc_set_F128C(): $Math::MPC::no_complex_c_q"}
+  my $t_re = Math::MPFR::Rmpfr_init2(113);
+  my $t_im = Math::MPFR::Rmpfr_init2(113);
+  Math::MPFR::Rmpfr_set_float128($t_re, Math::Complex_C::Q::real_cq($_[1]), 0); # Round to nearest
+  Math::MPFR::Rmpfr_set_float128($t_im, Math::Complex_C::Q::imag_cq($_[1]), 0);   # Round to nearest
+  return Rmpc_set_fr_fr($_[0], $t_re, $t_im, $_[2]); # Now use specified rounding mode
+}
+
+sub Rmpc_get_F128C {
+  if($Math::MPC::no_complex_c_q) {die "In Rmpc_set_F128C(): $Math::MPC::no_complex_c_q"}
+  my $t = Math::MPFR::Rmpfr_init2(113);
+  RMPC_RE($t, $_[1]);
+  Math::Complex_C::Q::set_real_cq($_[0], Math::MPFR::Rmpfr_get_float128($t, $_[2] & 3));
+  RMPC_IM($t, $_[1]);
+  Math::Complex_C::Q::set_imag_cq($_[0], Math::MPFR::Rmpfr_get_float128($t, $_[2] / 16));
+}
+
 
 sub new {
 
@@ -357,18 +383,9 @@ sub new {
     if($type2 == 0)
       {die "Second argument to new() is inappropriate"}
 
-    # Create a Math::MPC object that has $arg1 as its
-    # real component, and zero as its imaginary component.
-    my $mpc1 = _new_real($arg1);
-
-    # Create a Math::MPC object that has $arg2 as its
-    # imaginary component, and zero as its real component.
-    my $mpc2 = _new_im($arg2);
-
-    # Add the 2 created Math::MPC objects together and return
-    # the result
-    Rmpc_add($mpc1, $mpc1, $mpc2, MPC_RNDNN);
-    return $mpc1;
+    # Return a Math::MPC object that has $arg1 as its
+    # real component, and $arg2 as its imaginary component.
+    return _new_real_im($arg1, $arg2);
 }
 
 sub Rmpc_out_str {
@@ -422,6 +439,7 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
    The following documentation heavily plagiarises the mpc documentation.
 
    use warnings;
+   use Math::MPFR qw(:mpfr); # needed by some Math::MPC functions
    use Math::MPC qw(:mpc);
    Rmpc_set_default_prec(500); # Set default precision to 500 bits
    my $mpc1 = Math::MPC->new(12.5, 1125); # 12.5 + 1125*i
@@ -670,16 +688,21 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
    $si = Rmpc_set_q($rop, $mpq, $rnd);
    $si = Rmpc_set_z($rop, $mpz, $rnd);
    $si = Rmpc_set_fr($rop, $mpfr, $rnd);
-   $si = Rmpc_set_dc($rop, $cc, $rnd);
-   $si = Rmpc_set_ldc($rop, $lcc, $rnd);
+   $si = Rmpc_set_dc($rop, $cc, $rnd); # deprecated - use Rmpc_set_DC instead
+   $si = Rmpc_set_ldc($rop, $lcc, $rnd); # deprecated - use Rmpc_set_LDC instead
+   $si = Rmpc_set_DC($rop, $cc, $rnd);  # $cc is a Math:Complex_C object
+   $si = Rmpc_set_LDC($rop, $lcc, $rnd); # $lcc is a Math::Complex_C::L object
+   Rmpc_set_F128C($rop, $f128c, $rnd); # $f128c is a Math::Complex_C::Q object
     Set the value of $rop from 2nd arg, rounded to the precision of
     $rop towards the given direction $rnd.
     Don't use Rmpc_set_ld unless perl has been built with long
     double support. Don't use Rmpc_set_uj or Rmpc_set_sj unless
     perl has been built with long long int support.
-    For Rmpc_set_dc and Rmpc_set_ldc, an mpc library (version 0.9
-    or later) that has been built with support for these data types
-    is needed.
+    For Rmpc_set_DC and Rmpc_set_LDC, requires an mpc library (version 0.9
+    or later) that has been built with support for 'double _Complex' and 'long
+    double _Complex' data types (resp.). For Rmpc_set_F128C, the underlying
+    mpfr library needs to have been built with --enable-float128 option.
+
 
    $si = Rmpc_set_str($rop, $string, $base, $rnd);
    $si = Rmpc_strtoc($rop, $string, $base, $rnd);
@@ -1041,11 +1064,17 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
     assuming both output and input use rounding to nearest, will recover
     the original value of $op. See the mpc documentation for details.
 
-   Rmpc_get_dc($cc, $op, $rnd);
-   Rmpc_get_ldc($lcc, $op, $rnd);
-    Set the 'double _Complex'/'long double _Complex' object to the value
-    of $op, rounded according to $rnd. Needs an mpc library (version 0.9
-    or later) that has been built with support for these data types.
+   $si = Rmpc_get_dc($cc, $op, $rnd); # deprecated, use Rmpc_get_DC instead
+   $si = Rmpc_get_ldc($lcc, $op $rnd);# deprecated, use Rmpc_get_LDC instead
+   $si = Rmpc_get_DC($cc, $op, $rnd);  # $cc is a Math:Complex_C object
+   $si = Rmpc_get_LDC($lcc, $op, $rnd); # $lcc is a Math::Complex_C::L object
+   Rmpc_get_F128C($f128c, $op, $rnd);# $f128c is Math::Complex_C::Q object
+    Set the 'double _Complex'/'long double _Complex'/'__float128 _Complex'
+    object to the value of $op, rounded according to $rnd. For Rmpc_get_DC
+    and Rmpc_get_LDC, requires an mpc library (version 0.9 or later) that
+    has been built with support for 'double _Complex' and 'long double
+    _Complex' data types (resp.). For Rmpc_get_F128C, the underlying mpfr
+    library needs to have been built with --enable-float128 option.
 
    ####################
 
@@ -1063,7 +1092,7 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
     Math::MPC object with *current default precision*, and using
     the *current default rounding mode*.
 
-    Be aware also, that the sign of zero is not always handled
+    Be aware also, that the sign of zero may not always handled
     correctly by the overload subroutines. If it's important to you
     that the sign of zero be handled correctly, don't use the
     overloaded operators. (For multiplication, division, addition
@@ -1182,6 +1211,21 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
     used by the mpc library that Math::MPC uses.
     (The function is not exportable.)
 
+   $iv = Math::MPC::nok_pokflag(); # not exported
+    Returns the value of the nok_pok flag. This flag is
+    initialized to zero, but incemented by 1 whenever a
+    scalar that is both a float (NOK) and string (POK) is passed
+    to new() or to an overloaded operator. The value of the flag
+    therefore tells us how many times such events occurred . The
+    flag can be reset to 0 by running clear_nok_pok().
+
+   Math::MPC::set_nok_pok($iv); # not exported
+    Resets the nok_pok flag to the value specified by $iv.
+
+   Math::MPC::clear_nok_pok(); # not exported
+    Resets the nok_pok flag to 0.(Essentially the same as
+    running set_nok_pok(0).)
+
    ####################
 
 =head1 TODO
@@ -1197,14 +1241,14 @@ Math::MPC - perl interface to the MPC (multi precision complex) library.
     first thing to do is to check that the argument types
     you have supplied are appropriate.
     Also, as mentioned above in the "OPERATOR OVERLOADING" section,
-    the overloaded operators are not guaranteed to handle the sign
-    of zero correctly.
+    the overloaded operators are not guaranteed to always handle the
+    sign of zero correctly.
 
 =head1 LICENSE
 
     This program is free software; you may redistribute it and/or
     modify it under the same terms as Perl itself.
-    Copyright 2006-2009, 2010, 2011, 2014, Sisyphus
+    Copyright 2006-2011, 2014, 2016 Sisyphus
 
 =head1 AUTHOR
 
